@@ -188,6 +188,10 @@ namespace libtorrent {
 		// hidden
 		virtual ~plugin() {}
 
+#if TORRENT_ABI_VERSION == 1
+		using feature_flags_t = libtorrent::feature_flags_t;
+#endif
+
 		// include this bit if your plugin needs to alter the order of the
 		// optimistic unchoke of peers. i.e. have the on_optimistic_unchoke()
 		// callback be called.
@@ -277,6 +281,10 @@ namespace libtorrent {
 		// hidden
 		virtual ~torrent_plugin() {}
 
+#if TORRENT_ABI_VERSION == 1
+		using flags_t = libtorrent::add_peer_flags_t;
+#endif
+
 		// This function is called each time a new peer is connected to the torrent. You
 		// may choose to ignore this by just returning a default constructed
 		// ``shared_ptr`` (in which case you don't need to override this member
@@ -332,10 +340,6 @@ namespace libtorrent {
 		// enum members
 		virtual void on_state(torrent_status::state_t) {}
 
-		// called every time policy::add_peer is called
-		// src is a bitmask of which sources this peer
-		// has been seen from. flags is a bitmask of:
-
 		// this is the first time we see this peer
 		static constexpr add_peer_flags_t first_time = 1_bit;
 
