@@ -266,7 +266,7 @@ namespace libtorrent {
 		bool m_upload_mode:1;
 
 		// this is set to false as long as the connections
-		// of this torrent hasn't been initialized. If we
+		// of this torrent haven't been initialized. If we
 		// have metadata from the start, connections are
 		// initialized immediately, if we didn't have metadata,
 		// they are initialized right after files_checked().
@@ -1041,7 +1041,7 @@ namespace libtorrent {
 
 		torrent_handle get_handle();
 
-		void write_resume_data(add_torrent_params& atp) const;
+		void write_resume_data(resume_data_flags_t const flags, add_torrent_params& ret) const;
 
 		void seen_complete() { m_last_seen_complete = ::time(nullptr); }
 		int time_since_complete() const { return int(::time(nullptr) - m_last_seen_complete); }
@@ -1466,7 +1466,7 @@ namespace libtorrent {
 
 		// the number of bytes that has been
 		// downloaded that failed the hash-test
-		std::int32_t m_total_failed_bytes = 0;
+		std::int64_t m_total_failed_bytes = 0;
 		std::int64_t m_total_redundant_bytes = 0;
 
 		// the sequence number for this torrent, this is a
@@ -1476,7 +1476,7 @@ namespace libtorrent {
 		// used to post a message to defer disconnecting peers
 		std::vector<std::shared_ptr<peer_connection>> m_peers_to_disconnect;
 		aux::deferred_handler m_deferred_disconnect;
-		aux::handler_storage<96> m_deferred_handler_storage;
+		aux::handler_storage<104> m_deferred_handler_storage;
 
 		// these are the peer IDs we've used for our outgoing peer connections for
 		// this torrent. If we get an incoming peer claiming to have one of these,
@@ -1613,26 +1613,18 @@ namespace libtorrent {
 		// the maximum number of uploads for this torrent
 		std::uint32_t m_max_uploads:24;
 
-		// these are the flags sent in on a call to save_resume_data
-		// we need to save them to check them in write_resume_data
-		resume_data_flags_t m_save_resume_flags;
+		// 8 bits free
 
 // ----
 
 		// the number of unchoked peers in this torrent
 		unsigned int m_num_uploads:24;
 
-		// 3 unused bits
+		// 4 unused bits
 
 		// when this is true, this torrent supports peer exchange
 		bool m_enable_pex:1;
 
-		// this is set to true if the torrent was started without
-		// metadata. It is used to save metadata in the resume file
-		// by default for such torrents. It does not necessarily
-		// have to be a magnet link.
-		bool m_magnet_link:1;
-
 		// set to true if the session IP filter applies to this
 		// torrent or not. Defaults to true.
 		bool m_apply_ip_filter:1;
